@@ -22,7 +22,6 @@
 #include "downloadmanager.h"
 #include "mainapplication.h"
 #include "checkboxdialog.h"
-#include "widget.h"
 #include "qztools.h"
 #include "speeddial.h"
 #include "autofill.h"
@@ -42,6 +41,8 @@
 #include "ui_jsalert.h"
 #include "ui_jsprompt.h"
 #include "passwordmanager.h"
+#include "scripts.h"
+#include "ocssupport.h"
 
 #include <iostream>
 
@@ -57,10 +58,14 @@
 #include <QAuthenticator>
 #include <QPushButton>
 #include <QUrlQuery>
+#include <QtWebEngineWidgetsVersion>
+
+#include <QWebEngineRegisterProtocolHandlerRequest>
 
 QString WebPage::s_lastUploadLocation = QDir::homePath();
 QUrl WebPage::s_lastUnsupportedUrl;
 QTime WebPage::s_lastUnsupportedUrlTime;
+QStringList s_supportedSchemes;
 
 static const bool kEnableJsOutput = qEnvironmentVariableIsSet("FALKON_ENABLE_JS_OUTPUT");
 static const bool kEnableJsNonBlockDialogs = qEnvironmentVariableIsSet("FALKON_ENABLE_JS_NONBLOCK_DIALOGS");
@@ -73,7 +78,7 @@ WebPage::WebPage(QObject* parent)
     , m_blockAlerts(false)
     , m_secureStatus(false)
 {
-    QWebChannel *channel = new QWebChannel(this);
+    auto *channel = new QWebChannel(this);
     ExternalJsObject::setupWebChannel(channel, this);
     setWebChannel(channel, SafeJsWorld);
 
@@ -97,8 +102,7 @@ WebPage::WebPage(QObject* parent)
     m_contentsResizedConnection = connect(this, &QWebEnginePage::contentsSizeChanged, this, [this]() {
         const QString fragment = url().fragment();
         if (!fragment.isEmpty()) {
-            const QString src = QSL("var els = document.querySelectorAll(\"[name='%1']\"); if (els.length) els[0].scrollIntoView();");
-            runJavaScript(src.arg(fragment));
+            runJavaScript(Scripts::scrollToAnchor(fragment));
         }
         disconnect(m_contentsResizedConnection);
     });
@@ -106,13 +110,25 @@ WebPage::WebPage(QObject* parent)
     // Workaround for broken load started/finished signals in QtWebEngine 5.10, 5.11
     connect(this, &QWebEnginePage::loadProgress, this, [this](int progress) {
         if (progress == 100) {
-            emit loadFinished(true);
+            Q_EMIT loadFinished(true);
         }
     });
+
+    connect(this, &QWebEnginePage::registerProtocolHandlerRequested, this, [this](QWebEngineRegisterProtocolHandlerRequest request) {
+        delete m_registerProtocolHandlerRequest;
+        m_registerProtocolHandlerRequest = new QWebEngineRegisterProtocolHandlerRequest(request);
+    });
+    connect(this, &QWebEnginePage::printRequested, this, &WebPage::printRequested);
+    connect(this, &QWebEnginePage::selectClientCertificate, this, [this](QWebEngineClientCertificateSelection selection) {
+        // TODO: It should prompt user
+        selection.select(selection.certificates().at(0));
+    });
 }
 
 WebPage::~WebPage()
 {
+    delete m_registerProtocolHandlerRequest;
+
     if (m_runningLoop) {
         m_runningLoop->exit(1);
         m_runningLoop = 0;
@@ -193,6 +209,45 @@ bool WebPage::isLoading() const
     return m_loadProgress < 100;
 }
 
+// static
+QStringList WebPage::internalSchemes()
+{
+    return QStringList{
+        QSL("http"),
+        QSL("https"),
+        QSL("file"),
+        QSL("ftp"),
+        QSL("data"),
+        QSL("about"),
+        QSL("view-source"),
+        QSL("chrome")
+    };
+}
+
+// static
+QStringList WebPage::supportedSchemes()
+{
+    if (s_supportedSchemes.isEmpty()) {
+        s_supportedSchemes = internalSchemes();
+    }
+    return s_supportedSchemes;
+}
+
+// static
+void WebPage::addSupportedScheme(const QString &scheme)
+{
+    s_supportedSchemes = supportedSchemes();
+    if (!s_supportedSchemes.contains(scheme)) {
+        s_supportedSchemes.append(scheme);
+    }
+}
+
+// static
+void WebPage::removeSupportedScheme(const QString &scheme)
+{
+    s_supportedSchemes.removeOne(scheme);
+}
+
 void WebPage::urlChanged(const QUrl &url)
 {
     Q_UNUSED(url)
@@ -210,7 +265,7 @@ void WebPage::progress(int prog)
 
     if (secStatus != m_secureStatus) {
         m_secureStatus = secStatus;
-        emit privacyChanged(secStatus);
+        Q_EMIT privacyChanged(secStatus);
     }
 }
 
@@ -224,7 +279,7 @@ void WebPage::finished()
         if (info.isFile()) {
             if (!m_fileWatcher) {
                 m_fileWatcher = new DelayedFileWatcher(this);
-                connect(m_fileWatcher, SIGNAL(delayedFileChanged(QString)), this, SLOT(watchedFileChanged(QString)));
+                connect(m_fileWatcher, &DelayedFileWatcher::delayedFileChanged, this, &WebPage::watchedFileChanged);
             }
 
             const QString filePath = url().toLocalFile();
@@ -382,12 +437,18 @@ bool WebPage::acceptNavigationRequest(const QUrl &url, QWebEnginePage::Navigatio
             QUrlQuery query(url);
             mApp->searchEnginesManager()->addEngine(QUrl(query.queryItemValue(QSL("url"))));
             return false;
+#if QTWEBENGINEWIDGETS_VERSION < QT_VERSION_CHECK(5, 12, 0)
         } else if (url.path() == QL1S("PrintPage")) {
-            emit printRequested();
+            Q_EMIT printRequested();
             return false;
+#endif
         }
     }
 
+    if (url.scheme() == QL1S("ocs") && OcsSupport::instance()->handleUrl(url)) {
+        return false;
+    }
+
     const bool result = QWebEnginePage::acceptNavigationRequest(url, type, isMainFrame);
 
     if (result) {
@@ -396,7 +457,7 @@ bool WebPage::acceptNavigationRequest(const QUrl &url, QWebEnginePage::Navigatio
             const bool globalJsEnabled = mApp->webSettings()->testAttribute(QWebEngineSettings::JavascriptEnabled);
             settings()->setAttribute(QWebEngineSettings::JavascriptEnabled, isWeb ? globalJsEnabled : true);
         }
-        emit navigationRequestAccepted(url, type, isMainFrame);
+        Q_EMIT navigationRequestAccepted(url, type, isMainFrame);
     }
 
     return result;
@@ -441,6 +502,22 @@ QStringList WebPage::autoFillUsernames() const
     return m_autoFillUsernames;
 }
 
+QUrl WebPage::registerProtocolHandlerRequestUrl() const
+{
+    if (m_registerProtocolHandlerRequest && url().host() == m_registerProtocolHandlerRequest->origin().host()) {
+        return m_registerProtocolHandlerRequest->origin();
+    }
+    return {};
+}
+
+QString WebPage::registerProtocolHandlerRequestScheme() const
+{
+    if (m_registerProtocolHandlerRequest && url().host() == m_registerProtocolHandlerRequest->origin().host()) {
+        return m_registerProtocolHandlerRequest->scheme();
+    }
+    return {};
+}
+
 bool WebPage::javaScriptPrompt(const QUrl &securityOrigin, const QString &msg, const QString &defaultValue, QString* result)
 {
     if (!kEnableJsNonBlockDialogs) {
@@ -451,10 +528,10 @@ bool WebPage::javaScriptPrompt(const QUrl &securityOrigin, const QString &msg, c
         return false;
     }
 
-    ResizableFrame* widget = new ResizableFrame(view()->overlayWidget());
+    auto *widget = new QFrame(view()->overlayWidget());
 
     widget->setObjectName("jsFrame");
-    Ui_jsPrompt* ui = new Ui_jsPrompt();
+    auto* ui = new Ui_jsPrompt();
     ui->setupUi(widget);
     ui->message->setText(msg);
     ui->lineEdit->setText(defaultValue);
@@ -462,12 +539,17 @@ bool WebPage::javaScriptPrompt(const QUrl &securityOrigin, const QString &msg, c
     widget->resize(view()->size());
     widget->show();
 
-    connect(view(), SIGNAL(viewportResized(QSize)), widget, SLOT(slotResize(QSize)));
+    QAbstractButton *clicked = nullptr;
+    connect(ui->buttonBox, &QDialogButtonBox::clicked, this, [&](QAbstractButton *button) {
+        clicked = button;
+    });
+
+    connect(view(), &WebView::viewportResized, widget, QOverload<const QSize &>::of(&QFrame::resize));
     connect(ui->lineEdit, SIGNAL(returnPressed()), ui->buttonBox->button(QDialogButtonBox::Ok), SLOT(animateClick()));
 
     QEventLoop eLoop;
     m_runningLoop = &eLoop;
-    connect(ui->buttonBox, SIGNAL(clicked(QAbstractButton*)), &eLoop, SLOT(quit()));
+    connect(ui->buttonBox, &QDialogButtonBox::clicked, &eLoop, &QEventLoop::quit);
 
     if (eLoop.exec() == 1) {
         return result;
@@ -475,7 +557,7 @@ bool WebPage::javaScriptPrompt(const QUrl &securityOrigin, const QString &msg, c
     m_runningLoop = 0;
 
     QString x = ui->lineEdit->text();
-    bool _result = ui->buttonBox->clickedButtonRole() == QDialogButtonBox::AcceptRole;
+    bool _result = ui->buttonBox->buttonRole(clicked) == QDialogButtonBox::AcceptRole;
     *result = x;
 
     delete widget;
@@ -494,28 +576,33 @@ bool WebPage::javaScriptConfirm(const QUrl &securityOrigin, const QString &msg)
         return false;
     }
 
-    ResizableFrame* widget = new ResizableFrame(view()->overlayWidget());
+    auto *widget = new QFrame(view()->overlayWidget());
 
     widget->setObjectName("jsFrame");
-    Ui_jsConfirm* ui = new Ui_jsConfirm();
+    auto* ui = new Ui_jsConfirm();
     ui->setupUi(widget);
     ui->message->setText(msg);
     ui->buttonBox->button(QDialogButtonBox::Ok)->setFocus();
     widget->resize(view()->size());
     widget->show();
 
-    connect(view(), SIGNAL(viewportResized(QSize)), widget, SLOT(slotResize(QSize)));
+    QAbstractButton *clicked = nullptr;
+    connect(ui->buttonBox, &QDialogButtonBox::clicked, this, [&](QAbstractButton *button) {
+        clicked = button;
+    });
+
+    connect(view(), &WebView::viewportResized, widget, QOverload<const QSize &>::of(&QFrame::resize));
 
     QEventLoop eLoop;
     m_runningLoop = &eLoop;
-    connect(ui->buttonBox, SIGNAL(clicked(QAbstractButton*)), &eLoop, SLOT(quit()));
+    connect(ui->buttonBox, &QDialogButtonBox::clicked, &eLoop, &QEventLoop::quit);
 
     if (eLoop.exec() == 1) {
         return false;
     }
     m_runningLoop = 0;
 
-    bool result = ui->buttonBox->clickedButtonRole() == QDialogButtonBox::AcceptRole;
+    bool result = ui->buttonBox->buttonRole(clicked) == QDialogButtonBox::AcceptRole;
 
     delete widget;
     view()->setFocus();
@@ -549,21 +636,21 @@ void WebPage::javaScriptAlert(const QUrl &securityOrigin, const QString &msg)
         return;
     }
 
-    ResizableFrame* widget = new ResizableFrame(view()->overlayWidget());
+    auto *widget = new QFrame(view()->overlayWidget());
 
     widget->setObjectName("jsFrame");
-    Ui_jsAlert* ui = new Ui_jsAlert();
+    auto* ui = new Ui_jsAlert();
     ui->setupUi(widget);
     ui->message->setText(msg);
     ui->buttonBox->button(QDialogButtonBox::Ok)->setFocus();
     widget->resize(view()->size());
     widget->show();
 
-    connect(view(), SIGNAL(viewportResized(QSize)), widget, SLOT(slotResize(QSize)));
+    connect(view(), &WebView::viewportResized, widget, QOverload<const QSize &>::of(&QFrame::resize));
 
     QEventLoop eLoop;
     m_runningLoop = &eLoop;
-    connect(ui->buttonBox, SIGNAL(clicked(QAbstractButton*)), &eLoop, SLOT(quit()));
+    connect(ui->buttonBox, &QDialogButtonBox::clicked, &eLoop, &QEventLoop::quit);
 
     if (eLoop.exec() == 1) {
         return;
@@ -602,7 +689,7 @@ void WebPage::javaScriptConsoleMessage(JavaScriptConsoleMessageLevel level, cons
 
 QWebEnginePage* WebPage::createWindow(QWebEnginePage::WebWindowType type)
 {
-    TabbedWebView *tView = qobject_cast<TabbedWebView*>(view());
+    auto *tView = qobject_cast<TabbedWebView*>(view());
     BrowserWindow *window = tView ? tView->browserWindow() : mApp->getWindow();
 
     auto createTab = [=](Qz::NewTabPositionFlags pos) {
@@ -628,16 +715,16 @@ QWebEnginePage* WebPage::createWindow(QWebEnginePage::WebWindowType type)
     switch (type) {
     case QWebEnginePage::WebBrowserWindow: {
         BrowserWindow *window = mApp->createWindow(Qz::BW_NewWindow);
-        WebPage *page = new WebPage;
+        auto *page = new WebPage;
         window->setStartPage(page);
         return page;
     }
 
     case QWebEnginePage::WebDialog:
         if (!qzSettings->openPopupsInTabs) {
-            PopupWebView* view = new PopupWebView;
+            auto* view = new PopupWebView;
             view->setPage(new WebPage);
-            PopupWindow* popup = new PopupWindow(view);
+            auto* popup = new PopupWindow(view);
             popup->show();
             window->addDeleteOnCloseWidget(popup);
             return view->page();
@@ -654,5 +741,5 @@ QWebEnginePage* WebPage::createWindow(QWebEnginePage::WebWindowType type)
         break;
     }
 
-    return Q_NULLPTR;
+    return nullptr;
 }
